@@ -205,10 +205,17 @@ class coo_matrix(_data_matrix, _minmax_mixin):
         nrows, ncols = self.shape
 
         if order == 'C':
-            flat_indices = ncols * self.row + self.col
+            # Upcast to avoid overflows: the coo_matrix constructor
+            # below will downcast the results to a smaller dtype, if
+            # possible.
+            dtype = get_index_dtype(maxval=(ncols * max(0, nrows - 1) + max(0, ncols - 1)))
+
+            flat_indices = np.multiply(ncols, self.row, dtype=dtype) + self.col
             new_row, new_col = divmod(flat_indices, shape[1])
         elif order == 'F':
-            flat_indices = self.row + nrows * self.col
+            dtype = get_index_dtype(maxval=(nrows * max(0, ncols - 1) + max(0, nrows - 1)))
+
+            flat_indices = np.multiply(nrows, self.col, dtype=dtype) + self.row
             new_col, new_row = divmod(flat_indices, shape[0])
         else:
             raise ValueError("'order' must be 'C' or 'F'")
